@@ -4,19 +4,23 @@ use Mojo::Base -base;
 has fetch => 10;
 has [qw(minion options what)];
 
-sub next { shift @{shift->_fetch->{results}} }
+sub next { shift @{shift->_fetch(0)->{results}} }
+
+sub total { shift->_fetch(1)->{total} }
 
 sub _fetch {
-  my $self = shift;
+  my ($self, $lazy) = @_;
 
-  return $self if @{$self->{results} // []};
+  return $self if ($lazy && exists $self->{total}) || @{$self->{results} // []};
 
-  my $what    = $self->what;
+  my $what    = $self->{jobs} ? 'jobs' : 'workers';
   my $method  = "list_$what";
   my $options = $self->options;
   my $results = $self->minion->backend->$method(0, $self->fetch, $options);
 
-  push @{$self->{results}}, my @results = @{$results->{$what}};
+  $self->{total} = $results->{total} + ($self->{count} // 0);
+  $self->{count} += my @results = @{$results->{$what}};
+  push @{$self->{results}}, @results;
   $options->{before} = $results[-1]{id} if @results;
 
   return $self;
@@ -35,13 +39,7 @@ Minion::Iterator - Minion iterator
   use Minion::Iterator;
 
   my $iter = Minion::Iterator->new(
-    minion  => $minion,
-    options => {states => ['inactive']},
-    what    => 'jobs'
-  );
-  while (my $info = $iter->next) {
-    say $info->{id};
-  }
+    minion  => $minion, options => {states => ['inactive']});
 
 =head1 DESCRIPTION
 
@@ -74,13 +72,6 @@ L<Minion> object this job belongs to.
 Options to be passed to L<Minion::Backend/"list_jobs"> or
 L<Minion::Backend/"list_workers">.
 
-=head2 what
-
-  my $what = $iter->what;
-  $iter    = $iter->what('jobs');
-
-What to L</"fetch">.
-
 =head1 METHODS
 
 L<Minion::Iterator> inherits all methods from L<Mojo::Base> and implements the
@@ -92,6 +83,14 @@ following new ones.
 
 Get next value.
 
+=head2 total
+
+  my $num = $iter->total;
+
+Total number of results. If results are removed in the backend while iterating,
+this number will become an estimate that gets updated every time new results are
+fetched.
+
 =head1 SEE ALSO
 
 L<Minion>, L<Mojolicious::Guides>, L<https://mojolicious.org>.
